"use strict";
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataSourceRendererStatic = void 0;
const react_1 = __importStar(require("react"));
const DataSourceRendererVirtual_1 = require("./DataSourceRendererVirtual");
/**
 * This component is UI agnostic, and just takes care of rendering all items in the DataSource.
 * This component does not apply virtualization, so don't use it for large datasets!
 */
exports.DataSourceRendererStatic = (0, react_1.memo)(function DataSourceRendererStatic({ dataView, maxRecords, useFixedRowHeight, context, itemRenderer, onKeyDown, emptyRenderer, }) {
    /**
     * Virtualization
     */
    // render scheduling
    const [, setForceUpdate] = (0, react_1.useState)(0);
    const redraw = (0, react_1.useCallback)(() => {
        setForceUpdate((x) => x + 1);
    }, []);
    (0, react_1.useEffect)(function subscribeToDataSource() {
        let unmounted = false;
        dataView.setWindow(0, maxRecords);
        const unsubscribe = dataView.addListener((_event) => {
            if (unmounted) {
                return;
            }
            setForceUpdate((x) => x + 1);
        });
        return () => {
            unmounted = true;
            unsubscribe();
        };
    }, [dataView, maxRecords, setForceUpdate, useFixedRowHeight]);
    (0, react_1.useEffect)(() => {
        // initial virtualization is incorrect because the parent ref is not yet set, so trigger render after mount
        setForceUpdate((x) => x + 1);
    }, [setForceUpdate]);
    /**
     * Rendering
     */
    const records = dataView.output();
    if (records.length > 500) {
        console.warn("StaticDataSourceRenderer should only be used on small datasets. For large datasets the 'scrollable' flag should enabled on DataTable");
    }
    return (react_1.default.createElement(DataSourceRendererVirtual_1.RedrawContext.Provider, { value: redraw },
        react_1.default.createElement("div", { onKeyDown: onKeyDown, tabIndex: 0 }, records.length === 0
            ? emptyRenderer?.(dataView)
            : records.map((item, index) => (react_1.default.createElement("div", { key: index }, itemRenderer(item, index, context)))))));
});
//# sourceMappingURL=DataSourceRendererStatic.js.map