/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
import React from 'react';
import { HighlightManager } from './Highlight';
/**
 * A Formatter is used to render an arbitrarily value to React. If a formatter returns 'undefined'
 * it is considered a 'miss' and the next formatter will be tried, eventually falling back to the default formatter.
 *
 * In case further processing by the default formatter is to be avoided, make sure a string is returned from any custom formatter.
 */
export type Formatter = (value: any, highlighter?: HighlightManager) => string | React.ReactElement | any;
export declare const DataFormatter: {
    defaultFormatter(value: any, highlighter?: HighlightManager): {};
    truncate(maxLength: number): (value: any, highlighter?: HighlightManager) => any;
    /**
     * Formatter that will automatically create links for any urls inside the data
     */
    linkify(value: any): any;
    prettyPrintJson(value: any): any;
    format(value: any, formatters?: Formatter[] | Formatter, highlighter?: HighlightManager): any;
};
export declare function TruncateHelper({ value, maxLength, textWrapper, }: {
    value: string;
    maxLength: number;
    textWrapper?: HighlightManager;
}): JSX.Element;
//# sourceMappingURL=DataFormatter.d.ts.map