"use strict";
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withTrackingScope = exports.describeElement = exports.wrapInteractionHandler = exports.useTrackedCallback = exports.Tracked = exports.useCurrentScopeName = exports.TrackingScope = exports.TrackingScopeContext = exports.resetGlobalInteractionReporter = exports.setGlobalInteractionReporter = void 0;
const react_1 = __importStar(require("react"));
const react_2 = require("react");
const react_element_to_jsx_string_1 = __importDefault(require("react-element-to-jsx-string"));
const flipper_common_1 = require("flipper-common");
let globalInteractionReporter = () => { };
function setGlobalInteractionReporter(reporter) {
    globalInteractionReporter = reporter;
}
exports.setGlobalInteractionReporter = setGlobalInteractionReporter;
// For unit tests only
function resetGlobalInteractionReporter() {
    globalInteractionReporter = () => { };
}
exports.resetGlobalInteractionReporter = resetGlobalInteractionReporter;
const DEFAULT_SCOPE = 'Flipper';
exports.TrackingScopeContext = (0, react_2.createContext)(DEFAULT_SCOPE);
function TrackingScope({ scope, children, }) {
    const baseScope = (0, react_2.useContext)(exports.TrackingScopeContext);
    return (react_1.default.createElement(exports.TrackingScopeContext.Provider, { value: baseScope === DEFAULT_SCOPE ? scope : `${baseScope}:${scope}` }, children));
}
exports.TrackingScope = TrackingScope;
/**
 * Gives the name of the current scope that is currently rendering.
 * Typically the current plugin id, but can be further refined by using TrackingScopes
 */
function useCurrentScopeName() {
    return (0, react_2.useContext)(exports.TrackingScopeContext);
}
exports.useCurrentScopeName = useCurrentScopeName;
function Tracked({ events = 'onClick', children, action, }) {
    const scope = useCurrentScopeName();
    return react_2.Children.map(children, (child) => {
        if (!child || typeof child !== 'object') {
            return child;
        }
        if (child.type === Tracked)
            return child; // avoid double trapping
        const newProps = {};
        (typeof events === 'string' ? [events] : events).forEach((event) => {
            const base = child.props[event];
            if (!base) {
                return;
            }
            newProps[event] = wrapInteractionHandler(base, child, event, scope, action);
        });
        return (0, react_2.cloneElement)(child, newProps);
    });
}
exports.Tracked = Tracked;
function useTrackedCallback(action, fn, deps) {
    const scope = (0, react_2.useContext)(exports.TrackingScopeContext);
    return (0, react_1.useMemo)(() => {
        return wrapInteractionHandler(fn, null, '', scope, action);
        // eslint-disable-next-line
    }, deps);
}
exports.useTrackedCallback = useTrackedCallback;
function wrapInteractionHandler(fn, element, event, scope, action) {
    const interaction_uuid = (0, flipper_common_1.uuid)();
    function report(start, initialEnd, error) {
        const interactionReport = {
            duration: initialEnd - start,
            totalDuration: Date.now() - start,
            success: error ? 0 : 1,
            error: error ? `${error}` : undefined,
            componentType: element === null
                ? 'unknown'
                : typeof element === 'string'
                    ? element
                    : describeElementType(element),
            action: action ??
                (element && typeof element != 'string'
                    ? describeElement(element)
                    : 'unknown'),
            scope,
            event,
            uuid: interaction_uuid,
        };
        if (error && typeof error === 'object') {
            // associate the error with the interaction caused it
            error.interaction = interactionReport;
        }
        globalInteractionReporter(interactionReport);
    }
    const res = function trappedInteractionHandler() {
        let res;
        const start = Date.now();
        const r = report.bind(null, start);
        try {
            // eslint-disable-next-line
            res = fn.apply(this, arguments);
            if (Date.now() - start > 20) {
                console.warn('Slow interaction');
            }
        }
        catch (e) {
            r(Date.now(), e);
            throw e;
        }
        const initialEnd = Date.now();
        if (typeof res?.then === 'function' && typeof res?.catch === 'function') {
            // async / promise
            // eslint-disable-next-line promise/catch-or-return
            res.then(() => r(initialEnd), (error) => r(initialEnd, error));
            res = res.catch((error) => {
                // we need to create another rejected promise so error is again marked as "unhandled".
                return Promise.reject(error);
            });
        }
        else {
            // not a Promise
            r(initialEnd);
        }
        return res;
    };
    res.flipperTracked = true; // Avoid double wrapping / handling, if e.g. Button is wrapped in Tracked
    return res;
}
exports.wrapInteractionHandler = wrapInteractionHandler;
function describeElement(element) {
    const describing = element.props.title ?? element.props.children;
    if (typeof describing === 'string') {
        return describing;
    }
    if (typeof element.key === 'string') {
        return element.key;
    }
    return stringifyElement(element);
}
exports.describeElement = describeElement;
function describeElementType(element) {
    const t = element.type;
    // cases: native dom element, named (class) component, named function component, classname
    return typeof t === 'string'
        ? t
        : t?.displayName ?? t?.name ?? t?.constructor?.name ?? 'unknown';
}
function withTrackingScope(Component) {
    return function WithTrackingScope(props) {
        const scope = Component.displayName ?? Component.name ?? Component.constructor?.name;
        if (!scope || typeof scope !== 'string') {
            throw new Error('Failed to find component name for trackingScope');
        }
        return (react_1.default.createElement(TrackingScope, { scope: scope },
            react_1.default.createElement(Component, { ...props })));
    };
}
exports.withTrackingScope = withTrackingScope;
// @ts-ignore
global.FlipperTrackingScopeContext = exports.TrackingScopeContext;
//@ts-ignore
global.FlipperTracked = Tracked;
// @ts-ignore
global.flipperTrackInteraction = function flipperTrackInteraction(elementType, event, scope, action, fn, ...args) {
    // @ts-ignore
    if (fn.flipperTracked) {
        return fn(...args);
    }
    return wrapInteractionHandler(fn, elementType, event, scope, !action
        ? 'unknown action'
        : typeof action === 'string'
            ? action
            : stringifyElement(action))(...args);
};
function stringifyElement(element) {
    if (!element)
        return 'unknown element';
    if (typeof element === 'string')
        return element;
    if (Array.isArray(element))
        return element.filter(Boolean).map(stringifyElement).join('');
    return (0, react_element_to_jsx_string_1.default)(element).substr(0, 200).replace(/\n/g, ' ');
}
//# sourceMappingURL=Tracked.js.map