/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
import { DataInspectorExpanded } from './DataInspectorNode';
import { PureComponent, ReactElement } from 'react';
import React from 'react';
import { DataValueExtractor } from './DataPreview';
import { HighlightManager } from '../Highlight';
import { MenuProps } from 'antd';
export type DataInspectorProps = {
    /**
     * Object to inspect.
     */
    data: any;
    /**
     * Object to compare with the provided `data` property.
     * Differences will be styled accordingly in the UI.
     */
    diff?: any;
    /**
     * Whether to expand the root by default.
     */
    expandRoot?: boolean;
    /**
     * An optional callback that will explode a value into its type and value.
     * Useful for inspecting serialised data.
     */
    extractValue?: DataValueExtractor;
    /**
     * Callback when a value is edited.
     */
    setValue?: (path: Array<string>, val: any) => void;
    /**
     * Callback when a delete action is invoked.
     */
    onDelete?: (path: Array<string>) => void;
    /**
     * Render callback that can be used to customize the rendering of object keys.
     */
    onRenderName?: (path: Array<string>, name: string, highlighter: HighlightManager) => React.ReactElement;
    /**
     * Render callback that can be used to customize the rendering of object values.
     */
    onRenderDescription?: (description: React.ReactElement) => React.ReactElement;
    /**
     * Whether all objects and arrays should be collapsed by default.
     */
    collapsed?: boolean;
    /**
     * Object of all properties that will have tooltips
     */
    tooltips?: Object;
    /**
     * Filter nodes by some search text
     */
    filter?: string;
    /**
     * Highlight color of the search text
     */
    highlightColor?: string;
    /**
     * these should be ant design Menu.Item's
     */
    additionalContextMenuItems?: (parentPath: string[], value: any, name?: string) => ReactElement[];
};
type DataInspectorState = {
    expanded: DataInspectorExpanded;
    filterExpanded: DataInspectorExpanded;
    userExpanded: DataInspectorExpanded;
    filter: string;
    hoveredNodePath: string | undefined;
};
/**
 * Wrapper around `DataInspector` that handles expanded state.
 *
 * If you require lower level access to the state then use `DataInspector`
 * directly.
 */
export declare class DataInspector extends PureComponent<DataInspectorProps, DataInspectorState> {
    state: DataInspectorState;
    isContextMenuOpen: boolean;
    static getDerivedStateFromProps(nextProps: DataInspectorProps, currentState: DataInspectorState): {
        filterExpanded: Record<string, boolean>;
        expanded: {
            [x: string]: boolean;
        };
        filter: string;
    } | null;
    onExpanded: (path: string, isExpanded: boolean) => void;
    setHoveredNodePath: (path?: string) => void;
    removeHover: () => void;
    getRootData: () => any;
    render(): JSX.Element;
    getContextMenu: () => MenuProps;
}
export {};
//# sourceMappingURL=DataInspector.d.ts.map