/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
import { DataValueExtractor } from './DataPreview';
import React from 'react';
import { HighlightManager } from '../Highlight';
export { DataValueExtractor } from './DataPreview';
export declare const RootDataContext: React.Context<() => any>;
export type DataInspectorSetValue = (path: Array<string>, val: any) => void;
export type DataInspectorDeleteValue = (path: Array<string>) => void;
export type DataInspectorExpanded = {
    [key: string]: boolean;
};
export type DiffMetadataExtractor = (data: any, diff: any, key: string) => Array<{
    data: any;
    diff?: any;
    status?: 'added' | 'removed';
}>;
type DataInspectorProps = {
    /**
     * Object to inspect.
     */
    data: any;
    /**
     * Object to compare with the provided `data` property.
     * Differences will be styled accordingly in the UI.
     */
    diff?: any;
    /**
     * Current name of this value.
     */
    name?: string;
    /**
     * Current depth.
     */
    depth: number;
    /**
     * An array containing the current location of the data relative to its root.
     */
    parentPath: Array<string>;
    /**
     * Whether to expand the root by default.
     */
    expandRoot?: boolean;
    /**
     * An array of paths that are currently expanded.
     */
    expanded: DataInspectorExpanded;
    /**
     * An optional callback that will explode a value into its type and value.
     * Useful for inspecting serialised data.
     */
    extractValue?: DataValueExtractor;
    /**
     * Callback whenever the current expanded paths is changed.
     */
    onExpanded?: ((path: string, expanded: boolean) => void) | undefined | null;
    /**
    /**
     * Render callback that can be used to customize the rendering of object keys.
     */
    onRenderName?: (path: Array<string>, name: string, highlighter: HighlightManager) => React.ReactElement;
    /**
     * Render callback that can be used to customize the rendering of object values.
     */
    onRenderDescription?: (description: React.ReactElement) => React.ReactElement;
    /**
     * Callback when a value is edited.
     */
    setValue?: DataInspectorSetValue | undefined | null;
    /**
     * Whether all objects and arrays should be collapsed by default.
     */
    collapsed?: boolean;
    /**
     * Ancestry of parent objects, used to avoid recursive objects.
     */
    parentAncestry: Array<Object>;
    /**
     * Object of properties that will have tooltips
     */
    tooltips?: any;
    hoveredNodePath?: string;
    setHoveredNodePath: (path: string) => void;
};
/**
 * An expandable data inspector.
 *
 * This component is fairly low level. It's likely you're looking for
 * [`<ManagedDataInspector>`](#manageddatainspector).
 */
export declare const DataInspectorNode: React.FC<DataInspectorProps>;
//# sourceMappingURL=DataInspectorNode.d.ts.map