/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
import { OperatorConfig } from '../PowerSearch';
import { EnumLabels } from '../PowerSearch/PowerSearchConfig';
export type PowerSearchOperatorProcessor = (powerSearchOperatorConfig: OperatorConfig, searchValue: any, value: any) => boolean;
export declare const dataTablePowerSearchOperators: {
    string_matches_regex: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    string_contains: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    string_not_contains: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    string_matches_exactly: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    string_not_matches_exactly: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    searializable_object_contains: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    searializable_object_contains_any_of: () => {
        label: string;
        key: string;
        valueType: "STRING_SET";
    };
    searializable_object_matches_regex: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    searializable_object_not_contains: () => {
        label: string;
        key: string;
        valueType: "STRING";
    };
    searializable_object_contains_none_of: () => {
        label: string;
        key: string;
        valueType: "STRING_SET";
    };
    string_set_contains_any_of: () => {
        label: string;
        key: string;
        valueType: "STRING_SET";
    };
    string_set_contains_none_of: () => {
        label: string;
        key: string;
        valueType: "STRING_SET";
    };
    int_equals: () => {
        label: string;
        key: string;
        valueType: "INTEGER";
    };
    int_greater_than: () => {
        label: string;
        key: string;
        valueType: "INTEGER";
    };
    int_greater_or_equal: () => {
        label: string;
        key: string;
        valueType: "INTEGER";
    };
    int_less_than: () => {
        label: string;
        key: string;
        valueType: "INTEGER";
    };
    int_less_or_equal: () => {
        label: string;
        key: string;
        valueType: "INTEGER";
    };
    float_equals: (precision?: number) => {
        label: string;
        key: string;
        valueType: "FLOAT";
        precision: number | undefined;
    };
    float_greater_than: () => {
        label: string;
        key: string;
        valueType: "FLOAT";
    };
    float_greater_or_equal: () => {
        label: string;
        key: string;
        valueType: "FLOAT";
    };
    float_less_than: () => {
        label: string;
        key: string;
        valueType: "FLOAT";
    };
    float_less_or_equal: () => {
        label: string;
        key: string;
        valueType: "FLOAT";
    };
    enum_set_is_nullish_or_any_of: (enumLabels: EnumLabels, allowFreeform?: boolean) => {
        label: string;
        key: string;
        valueType: "ENUM_SET";
        enumLabels: EnumLabels;
        allowFreeform: boolean | undefined;
    };
    enum_set_is_any_of: (enumLabels: EnumLabels, allowFreeform?: boolean) => {
        label: string;
        key: string;
        valueType: "ENUM_SET";
        enumLabels: EnumLabels;
        allowFreeform: boolean | undefined;
    };
    enum_set_is_none_of: (enumLabels: EnumLabels, allowFreeform?: boolean) => {
        label: string;
        key: string;
        valueType: "ENUM_SET";
        enumLabels: EnumLabels;
        allowFreeform: boolean | undefined;
    };
    is_nullish: () => {
        label: string;
        key: string;
        valueType: "NO_VALUE";
    };
    newer_than_absolute_date: () => {
        key: string;
        label: string;
        valueType: "ABSOLUTE_DATE";
        dateOnly: false;
    };
    newer_than_absolute_date_no_time: () => {
        key: string;
        label: string;
        valueType: "ABSOLUTE_DATE";
        dateOnly: true;
    };
    older_than_absolute_date: () => {
        key: string;
        label: string;
        valueType: "ABSOLUTE_DATE";
        dateOnly: false;
    };
    older_than_absolute_date_no_time: () => {
        key: string;
        label: string;
        valueType: "ABSOLUTE_DATE";
        dateOnly: true;
    };
    same_as_absolute_date_no_time: () => {
        key: string;
        label: string;
        valueType: "ABSOLUTE_DATE";
        dateOnly: true;
    };
};
export type PowerSearchOperatorProcessorConfig = {
    [K in keyof typeof dataTablePowerSearchOperators]: PowerSearchOperatorProcessor;
};
export declare const dataTablePowerSearchOperatorProcessorConfig: {
    string_matches_regex: (_operator: OperatorConfig, searchValue: string, value: string) => boolean;
    string_contains: (_operator: OperatorConfig, searchValue: string, value: string) => boolean;
    string_not_contains: (_operator: OperatorConfig, searchValue: string, value: string) => boolean;
    searializable_object_matches_regex: (_operator: OperatorConfig, searchValue: string, value: object) => boolean;
    searializable_object_contains: (_operator: OperatorConfig, searchValue: string, value: object) => boolean;
    searializable_object_contains_any_of: (_operator: OperatorConfig, searchValue: string[], value: object) => boolean;
    searializable_object_not_contains: (_operator: OperatorConfig, searchValue: string, value: object) => boolean;
    searializable_object_contains_none_of: (_operator: OperatorConfig, searchValue: string[], value: object) => boolean;
    string_matches_exactly: (_operator: OperatorConfig, searchValue: string, value: string) => boolean;
    string_not_matches_exactly: (_operator: OperatorConfig, searchValue: string, value: string) => boolean;
    string_set_contains_any_of: (_operator: OperatorConfig, searchValue: string[], value: string) => boolean;
    string_set_contains_none_of: (_operator: OperatorConfig, searchValue: string[], value: string) => boolean;
    int_equals: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    int_greater_than: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    int_greater_or_equal: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    int_less_than: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    int_less_or_equal: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    float_equals: (operator: OperatorConfig, searchValue: number, value: number) => boolean;
    float_greater_than: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    float_greater_or_equal: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    float_less_than: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    float_less_or_equal: (_operator: OperatorConfig, searchValue: number, value: number) => boolean;
    enum_set_is_nullish_or_any_of: (_operator: OperatorConfig, searchValue: string[], value?: string | null) => boolean;
    enum_set_is_any_of: (_operator: OperatorConfig, searchValue: string[], value: string) => boolean;
    enum_set_is_none_of: (_operator: OperatorConfig, searchValue: string[], value: string) => boolean;
    is_nullish: (_operator: OperatorConfig, _searchValue: any, value: any) => boolean;
    newer_than_absolute_date: (_operator: OperatorConfig, searchValue: Date, value: any) => boolean;
    newer_than_absolute_date_no_time: (_operator: OperatorConfig, searchValue: Date, value: any) => boolean;
    older_than_absolute_date: (_operator: OperatorConfig, searchValue: Date, value: any) => boolean;
    older_than_absolute_date_no_time: (_operator: OperatorConfig, searchValue: Date, value: any) => boolean;
    same_as_absolute_date_no_time: (_operator: OperatorConfig, searchValue: Date, value: any) => boolean;
};
//# sourceMappingURL=DataTableDefaultPowerSearchOperators.d.ts.map