"use strict";
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMemoize = void 0;
const react_1 = require("react");
/**
 * Slight variation on useMemo that encourages to create hoistable memoization functions,
 * which encourages reuse and testability by no longer closing over things in the memoization function.
 *
 * @example
 * const metroDevice = useMemoize(
 *   findMetroDevice,
 *   [connections.devices],
 * );
 */
function useMemoize(fn, args) {
    // eslint-disable-next-line
    return (0, react_1.useMemo)(() => fn.apply(null, args), args);
}
exports.useMemoize = useMemoize;
//# sourceMappingURL=useMemoize.js.map